# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Kustomize(UniversalBaseModel):
    patch: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Content of kustomization.yaml to perform kustomize operation. Please do not include the `resources` section. It is filled in automatically
    """

    additions: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field(default=None)
    """
    Additional kubernetes manifests to be included in the application
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class HuggingfaceArtifactSource(UniversalBaseModel):
    """
    Input for Artifact from Huggingface Model Hub
    """

    type: typing.Literal["huggingface-hub"] = pydantic.Field(default="huggingface-hub")
    """
    +value=huggingface-hub
    """

    model_id: str = pydantic.Field()
    """
    Model ID of the artifact to be downloaded
    """

    revision: str = pydantic.Field()
    """
    Revision of the artifact to be downloaded
    """

    ignore_patterns: typing.List[str] = pydantic.Field()
    """
    List of patterns to ignore while downloading the artifact
    """

    download_path_env_variable: str = pydantic.Field()
    """
    Environment variable which will contain the download path of the artifact
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .email import Email
from .slack_bot import SlackBot
from .slack_webhook import SlackWebhook

NotificationTarget = typing.Union[Email, SlackWebhook, SlackBot]



# This file was auto-generated by Fern from our API Definition.

import typing

from .async_service import AsyncService
from .helm import Helm
from .job import Job
from .service import Service

ApplicationSetComponentsItem = typing.Union[Service, AsyncService, Job, Helm]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class OciRepo(UniversalBaseModel):
    """
    OCIRepo
    """

    type: typing.Literal["oci-repo"] = pydantic.Field(default="oci-repo")
    """
    +value=oci-repo
    """

    oci_chart_url: str = pydantic.Field()
    """
    OCI chart URL - Need to be a valid URL.
    """

    integration_fqn: typing.Optional[str] = pydantic.Field(default=None)
    """
    FQN of the container registry. If you can't find your registry here,
    add it through the [Integrations](/integrations) page
    """

    version: str = pydantic.Field()
    """
    Helm chart version
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .worker_config_input_config import WorkerConfigInputConfig
from .worker_config_output_config import WorkerConfigOutputConfig


class WorkerConfig(UniversalBaseModel):
    input_config: WorkerConfigInputConfig = pydantic.Field()
    """
    Input Config
    """

    output_config: typing.Optional[WorkerConfigOutputConfig] = pydantic.Field(default=None)
    """
    Output Config
    """

    num_concurrent_workers: int = pydantic.Field(default=1)
    """
    Number of concurrent workers to spawn for the processor
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.query_spans_response import QuerySpansResponse
from ..types.sort_direction import SortDirection
from ..types.subject_type import SubjectType
from ..types.trace_span import TraceSpan
from .raw_client import AsyncRawTracesClient, RawTracesClient
from .types.query_spans_request_filters_item import QuerySpansRequestFiltersItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TracesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTracesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTracesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTracesClient
        """
        return self._raw_client

    def query_spans(
        self,
        *,
        start_time: str,
        tracing_project_fqn: str,
        end_time: typing.Optional[str] = OMIT,
        trace_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        span_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        parent_span_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        created_by_subject_types: typing.Optional[typing.Sequence[SubjectType]] = OMIT,
        created_by_subject_slugs: typing.Optional[typing.Sequence[str]] = OMIT,
        application_names: typing.Optional[typing.Sequence[str]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort_direction: typing.Optional[SortDirection] = OMIT,
        page_token: typing.Optional[str] = OMIT,
        filters: typing.Optional[typing.Sequence[QuerySpansRequestFiltersItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[TraceSpan, QuerySpansResponse]:
        """
        Parameters
        ----------
        start_time : str
            Start time in ISO 8601 format (e.g., 2025-03-12T00:00:09.872Z)

        tracing_project_fqn : str
            Tracing project FQN (e.g., truefoundry:tracing-project:tfy-default)

        end_time : typing.Optional[str]
            End time in ISO 8601 format (e.g., 2025-03-12T00:10:00.000Z). Defaults to current time if not provided.

        trace_ids : typing.Optional[typing.Sequence[str]]
            Array of trace IDs to filter by

        span_ids : typing.Optional[typing.Sequence[str]]
            Array of span IDs to filter by

        parent_span_ids : typing.Optional[typing.Sequence[str]]
            Array of parent span IDs to filter by

        created_by_subject_types : typing.Optional[typing.Sequence[SubjectType]]
            Array of subject types to filter by

        created_by_subject_slugs : typing.Optional[typing.Sequence[str]]
            Array of subject slugs to filter by

        application_names : typing.Optional[typing.Sequence[str]]
            Array of application names to filter by

        limit : typing.Optional[int]
            The maximum number of spans to return per page. Defaults to 200 if not provided.

        sort_direction : typing.Optional[SortDirection]
            Sort direction for results based on time. Defaults to descending (latest first)

        page_token : typing.Optional[str]
            An opaque string that should be passed as-is from previous response for fetching the next page. Pass `$response.pagination.nextPageToken` from previous response for fetching the next page.

        filters : typing.Optional[typing.Sequence[QuerySpansRequestFiltersItem]]
            Array of filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[TraceSpan, QuerySpansResponse]
            Returns all the spans matching the query.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.traces.query_spans(
            start_time="startTime",
            tracing_project_fqn="tracingProjectFqn",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.query_spans(
            start_time=start_time,
            tracing_project_fqn=tracing_project_fqn,
            end_time=end_time,
            trace_ids=trace_ids,
            span_ids=span_ids,
            parent_span_ids=parent_span_ids,
            created_by_subject_types=created_by_subject_types,
            created_by_subject_slugs=created_by_subject_slugs,
            application_names=application_names,
            limit=limit,
            sort_direction=sort_direction,
            page_token=page_token,
            filters=filters,
            request_options=request_options,
        )


class AsyncTracesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTracesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTracesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTracesClient
        """
        return self._raw_client

    async def query_spans(
        self,
        *,
        start_time: str,
        tracing_project_fqn: str,
        end_time: typing.Optional[str] = OMIT,
        trace_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        span_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        parent_span_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        created_by_subject_types: typing.Optional[typing.Sequence[SubjectType]] = OMIT,
        created_by_subject_slugs: typing.Optional[typing.Sequence[str]] = OMIT,
        application_names: typing.Optional[typing.Sequence[str]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort_direction: typing.Optional[SortDirection] = OMIT,
        page_token: typing.Optional[str] = OMIT,
        filters: typing.Optional[typing.Sequence[QuerySpansRequestFiltersItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[TraceSpan, QuerySpansResponse]:
        """
        Parameters
        ----------
        start_time : str
            Start time in ISO 8601 format (e.g., 2025-03-12T00:00:09.872Z)

        tracing_project_fqn : str
            Tracing project FQN (e.g., truefoundry:tracing-project:tfy-default)

        end_time : typing.Optional[str]
            End time in ISO 8601 format (e.g., 2025-03-12T00:10:00.000Z). Defaults to current time if not provided.

        trace_ids : typing.Optional[typing.Sequence[str]]
            Array of trace IDs to filter by

        span_ids : typing.Optional[typing.Sequence[str]]
            Array of span IDs to filter by

        parent_span_ids : typing.Optional[typing.Sequence[str]]
            Array of parent span IDs to filter by

        created_by_subject_types : typing.Optional[typing.Sequence[SubjectType]]
            Array of subject types to filter by

        created_by_subject_slugs : typing.Optional[typing.Sequence[str]]
            Array of subject slugs to filter by

        application_names : typing.Optional[typing.Sequence[str]]
            Array of application names to filter by

        limit : typing.Optional[int]
            The maximum number of spans to return per page. Defaults to 200 if not provided.

        sort_direction : typing.Optional[SortDirection]
            Sort direction for results based on time. Defaults to descending (latest first)

        page_token : typing.Optional[str]
            An opaque string that should be passed as-is from previous response for fetching the next page. Pass `$response.pagination.nextPageToken` from previous response for fetching the next page.

        filters : typing.Optional[typing.Sequence[QuerySpansRequestFiltersItem]]
            Array of filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[TraceSpan, QuerySpansResponse]
            Returns all the spans matching the query.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.traces.query_spans(
                start_time="startTime",
                tracing_project_fqn="tracingProjectFqn",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.query_spans(
            start_time=start_time,
            tracing_project_fqn=tracing_project_fqn,
            end_time=end_time,
            trace_ids=trace_ids,
            span_ids=span_ids,
            parent_span_ids=parent_span_ids,
            created_by_subject_types=created_by_subject_types,
            created_by_subject_slugs=created_by_subject_slugs,
            application_names=application_names,
            limit=limit,
            sort_direction=sort_direction,
            page_token=page_token,
            filters=filters,
            request_options=request_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from .cloudera_token_auth import ClouderaTokenAuth
from .cloudera_workload_auth_token import ClouderaWorkloadAuthToken

ClouderaProviderAccountAuthData = typing.Union[ClouderaWorkloadAuthToken, ClouderaTokenAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .fiddler_guard_type import FiddlerGuardType
from .fiddler_key_auth import FiddlerKeyAuth


class FiddlerGuardrailConfig(UniversalBaseModel):
    """
    Fiddler
    """

    name: str = pydantic.Field()
    """
    The name of the Guardrail Config.
    """

    type: typing.Literal["integration/guardrail-config/fiddler"] = pydantic.Field(
        default="integration/guardrail-config/fiddler"
    )
    """
    +uiType=Hidden
    +value=integration/guardrail-config/fiddler
    """

    auth_data: FiddlerKeyAuth
    guard_type: FiddlerGuardType
    operation: typing.Optional[typing.Literal["validate"]] = pydantic.Field(default=None)
    """
    The operation type for this guardrail. Fiddler guardrails can only be used for validation.
    """

    threshold: typing.Optional[float] = pydantic.Field(default=None)
    """
    Confidence threshold (0.0-1.0) for flagging content. Content scoring above the threshold will be flagged
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EnkryptAiKeyAuth(UniversalBaseModel):
    """
    Authentication method using Enkrypt AI API key
    """

    type: typing.Literal["api-key"] = pydantic.Field(default="api-key")
    """
    +value=api-key
    """

    api_key: str = pydantic.Field()
    """
    The API key for Enkrypt AI authentication
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SsoTeamManifest(UniversalBaseModel):
    type: typing.Literal["sso-team"] = pydantic.Field(default="sso-team")
    """
    +value=sso-team
    """

    name: str = pydantic.Field()
    """
    Name of the Team
    """

    sso_name: str = pydantic.Field()
    """
    SSO configuration to use for this team
    """

    group_name: str = pydantic.Field()
    """
    Group name from SSO that will be used to determine team membership
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .ai21provider_account import Ai21ProviderAccount
from .anthropic_provider_account import AnthropicProviderAccount
from .aws_bedrock_provider_account import AwsBedrockProviderAccount
from .aws_sagemaker_provider_account import AwsSagemakerProviderAccount
from .azure_foundry_provider_account import AzureFoundryProviderAccount
from .azure_open_ai_provider_account import AzureOpenAiProviderAccount
from .cerebras_provider_account import CerebrasProviderAccount
from .cloudera_provider_account import ClouderaProviderAccount
from .cohere_provider_account import CohereProviderAccount
from .databricks_provider_account import DatabricksProviderAccount
from .deepinfra_provider_account import DeepinfraProviderAccount
from .google_gemini_provider_account import GoogleGeminiProviderAccount
from .google_vertex_provider_account import GoogleVertexProviderAccount
from .groq_provider_account import GroqProviderAccount
from .mistral_ai_provider_account import MistralAiProviderAccount
from .nomic_provider_account import NomicProviderAccount
from .ollama_provider_account import OllamaProviderAccount
from .open_router_provider_account import OpenRouterProviderAccount
from .openai_provider_account import OpenaiProviderAccount
from .palm_provider_account import PalmProviderAccount
from .perplexity_ai_provider_account import PerplexityAiProviderAccount
from .samba_nova_provider_account import SambaNovaProviderAccount
from .self_hosted_model_provider_account import SelfHostedModelProviderAccount
from .snowflake_cortex_provider_account import SnowflakeCortexProviderAccount
from .together_ai_provider_account import TogetherAiProviderAccount
from .virtual_model_provider_account import VirtualModelProviderAccount

ModelProviderAccount = typing.Union[
    AwsBedrockProviderAccount,
    GoogleVertexProviderAccount,
    GoogleGeminiProviderAccount,
    AzureOpenAiProviderAccount,
    AzureFoundryProviderAccount,
    CohereProviderAccount,
    Ai21ProviderAccount,
    AnthropicProviderAccount,
    DeepinfraProviderAccount,
    GroqProviderAccount,
    MistralAiProviderAccount,
    PerplexityAiProviderAccount,
    TogetherAiProviderAccount,
    NomicProviderAccount,
    PalmProviderAccount,
    OllamaProviderAccount,
    OpenaiProviderAccount,
    DatabricksProviderAccount,
    ClouderaProviderAccount,
    SelfHostedModelProviderAccount,
    OpenRouterProviderAccount,
    SambaNovaProviderAccount,
    AwsSagemakerProviderAccount,
    CerebrasProviderAccount,
    SnowflakeCortexProviderAccount,
    VirtualModelProviderAccount,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_severity import AlertSeverity


class PrometheusAlertRule(UniversalBaseModel):
    """
    Prometheus Alert Rule
    """

    type: typing.Literal["prometheus-alert-rule"] = pydantic.Field(default="prometheus-alert-rule")
    """
    Type
    """

    name: str = pydantic.Field()
    """
    Enter a unique, descriptive name for your Prometheus alert rule. Choose a name that clearly indicates the condition being monitored. Please note that reserved alert names for TrueFoundry managed alerts are not allowed. For checking the list of reserved alert names, please refer to the documentation.
    """

    expression: str = pydantic.Field()
    """
    Enter a valid PromQL expression that defines the condition for triggering this alert. The alert will fire when this expression evaluates to true for the duration specified in the duration to trigger alert field.
    """

    for_: typing_extensions.Annotated[float, FieldMetadata(alias="for")] = pydantic.Field()
    """
    The prometheus expression must remain true for this duration (in seconds) before the alert is triggered. If the condition becomes false before this time elapses, the alert will not fire.
    """

    severity: AlertSeverity
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the alert rule which will be displayed in the alert rule list. This can be used to provide more context about the alert rule.
    """

    notification_enabled: bool = pydantic.Field(default=True)
    """
    When enabled, notifications will be sent to all configured target channels when the alert conditions are met.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .policy_actions import PolicyActions
from .policy_entity_types import PolicyEntityTypes
from .policy_filters import PolicyFilters
from .policy_manifest_mode import PolicyManifestMode
from .policy_manifest_operation import PolicyManifestOperation


class PolicyManifest(UniversalBaseModel):
    type: typing.Literal["policy"] = pydantic.Field(default="policy")
    """
    +value=policy
    """

    name: str = pydantic.Field()
    """
    Unique identifier of the policy across the organisation
    """

    description: str = pydantic.Field()
    """
    Description of the policy that explains what the policy does, its purpose, and how it affects resources. This helps other users understand the policy's behavior and impact.
    """

    operation: PolicyManifestOperation = pydantic.Field()
    """
    Defines if the policy validates or mutates resources. Validation policies check TrueFoundry manifests to ensure they meet conditions and can block deployments. Mutation policies modify Kubernetes manifests before they're applied to the cluster. See this [documentation](https://docs.truefoundry.com/docs/applying-custom-policies#applying-custom-policies) for more details.
    """

    mode: PolicyManifestMode = pydantic.Field()
    """
    Mode of the policy: `Audit` logs all policy evaluations without blocking deployments. `Enforce` blocks deployments if the policy fails. `Disabled` deactivates the policy.
    """

    entities: typing.List[PolicyEntityTypes] = pydantic.Field()
    """
    Types of applications this policy applies to
    """

    actions: typing.List[PolicyActions] = pydantic.Field()
    """
    Actions this policy applies to
    """

    filters: typing.Optional[PolicyFilters] = None
    code: str = pydantic.Field()
    """
    TypeScript code for Policy logic. To write and test your policy code, please refer to this [repository](https://github.com/truefoundry/tfy-typescript-policy)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .webhook_basic_auth import WebhookBasicAuth
from .webhook_bearer_auth import WebhookBearerAuth

WebhookIntegrationAuthData = typing.Union[WebhookBasicAuth, WebhookBearerAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_bedrock_guardrail_config import AwsBedrockGuardrailConfig
from .azure_content_safety_guardrail_config import AzureContentSafetyGuardrailConfig
from .azure_pii_guardrail_config import AzurePiiGuardrailConfig
from .custom_guardrail_config import CustomGuardrailConfig
from .enkrypt_ai_guardrail_config import EnkryptAiGuardrailConfig
from .fiddler_guardrail_config import FiddlerGuardrailConfig
from .open_ai_moderations_guardrail_config import OpenAiModerationsGuardrailConfig
from .palo_alto_prisma_airs_guardrail_config import PaloAltoPrismaAirsGuardrailConfig
from .pangea_guardrail_config import PangeaGuardrailConfig
from .patronus_guardrail_config import PatronusGuardrailConfig
from .prompt_foo_guardrail_config import PromptFooGuardrailConfig

GuardrailConfigIntegrations = typing.Union[
    OpenAiModerationsGuardrailConfig,
    AwsBedrockGuardrailConfig,
    CustomGuardrailConfig,
    AzurePiiGuardrailConfig,
    AzureContentSafetyGuardrailConfig,
    EnkryptAiGuardrailConfig,
    PaloAltoPrismaAirsGuardrailConfig,
    PromptFooGuardrailConfig,
    FiddlerGuardrailConfig,
    PangeaGuardrailConfig,
    PatronusGuardrailConfig,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class UserMetadataTenantRoleManagedBy(enum.StrEnum):
    MANUAL = "manual"
    GROUP = "group"
    INITIAL = "initial"

    def visit(
        self,
        manual: typing.Callable[[], T_Result],
        group: typing.Callable[[], T_Result],
        initial: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is UserMetadataTenantRoleManagedBy.MANUAL:
            return manual()
        if self is UserMetadataTenantRoleManagedBy.GROUP:
            return group()
        if self is UserMetadataTenantRoleManagedBy.INITIAL:
            return initial()



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.job_trigger_input import JobTriggerInput
from ...types.spark_job_trigger_input import SparkJobTriggerInput

TriggerJobRequestInput = typing.Union[JobTriggerInput, SparkJobTriggerInput]



# This file was auto-generated by Fern from our API Definition.

import typing

from .databricks_api_key_auth import DatabricksApiKeyAuth
from .databricks_service_principal_auth import DatabricksServicePrincipalAuth

DatabricksProviderAccountAuthData = typing.Union[DatabricksServicePrincipalAuth, DatabricksApiKeyAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

from .cpu_utilization_metric import CpuUtilizationMetric
from .cron_metric import CronMetric
from .rps_metric import RpsMetric

ServiceAutoscalingMetrics = typing.Union[CpuUtilizationMetric, RpsMetric, CronMetric]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alert_config_resource_type import AlertConfigResourceType


class AlertConfigResource(UniversalBaseModel):
    type: AlertConfigResourceType = pydantic.Field()
    """
    The type of resource this alert config belongs to. Can be either an application or a cluster.
    """

    fqn: str = pydantic.Field()
    """
    FQN of the resource this alert config belongs to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .sqs_queue_metric_config_auth import SqsQueueMetricConfigAuth


class SqsQueueMetricConfig(UniversalBaseModel):
    type: typing.Literal["sqs"] = pydantic.Field(default="sqs")
    """
    +value=sqs
    """

    queue_length: int = pydantic.Field()
    """
    Upper limit of the number of backlog messages the auto-scaler will try to maintain per replica. If you set this number to 10 and have 30 messages in the queue and one replica, the auto-scaler will scale the number of replicas to 3.
    """

    auth: typing.Optional[SqsQueueMetricConfigAuth] = pydantic.Field(default=None)
    """
    Receive permissions on the SQS Queue
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class AlertStatus(enum.StrEnum):
    FIRING = "firing"
    RESOLVED = "resolved"

    def visit(self, firing: typing.Callable[[], T_Result], resolved: typing.Callable[[], T_Result]) -> T_Result:
        if self is AlertStatus.FIRING:
            return firing()
        if self is AlertStatus.RESOLVED:
            return resolved()



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class AiGatewayGetGatewayConfigRequestType(enum.StrEnum):
    GATEWAY_RATE_LIMITING_CONFIG = "gateway-rate-limiting-config"
    GATEWAY_FALLBACK_CONFIG = "gateway-fallback-config"
    GATEWAY_LOAD_BALANCING_CONFIG = "gateway-load-balancing-config"
    GATEWAY_GUARDRAILS_CONFIG = "gateway-guardrails-config"
    GATEWAY_BUDGET_CONFIG = "gateway-budget-config"
    GATEWAY_OTEL_CONFIG = "gateway-otel-config"

    def visit(
        self,
        gateway_rate_limiting_config: typing.Callable[[], T_Result],
        gateway_fallback_config: typing.Callable[[], T_Result],
        gateway_load_balancing_config: typing.Callable[[], T_Result],
        gateway_guardrails_config: typing.Callable[[], T_Result],
        gateway_budget_config: typing.Callable[[], T_Result],
        gateway_otel_config: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_RATE_LIMITING_CONFIG:
            return gateway_rate_limiting_config()
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_FALLBACK_CONFIG:
            return gateway_fallback_config()
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_LOAD_BALANCING_CONFIG:
            return gateway_load_balancing_config()
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_GUARDRAILS_CONFIG:
            return gateway_guardrails_config()
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_BUDGET_CONFIG:
            return gateway_budget_config()
        if self is AiGatewayGetGatewayConfigRequestType.GATEWAY_OTEL_CONFIG:
            return gateway_otel_config()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class WorkbenchImage(UniversalBaseModel):
    """
    Workbench Image with persistent environment (Python 3.11.6)
    """

    image_uri: str = pydantic.Field()
    """
    The image URI. Specify the name of the image and the tag.
    If the image is in Dockerhub, you can skip registry-url (for e.g. `tensorflow/tensorflow`).
    You can use an image from a private registry using Advanced fields
    """

    build_script: typing.Optional[str] = pydantic.Field(default=None)
    """
    The build script to run when building the image.
    This will be executed as the last step in the docker build process as the root user (RUN DEBIAN_FRONTEND=noninteractive bash -ex build_script.sh)
    """

    docker_registry: typing.Optional[str] = pydantic.Field(default=None)
    """
    FQN of the container registry. If you can't find your registry here,
    add it through the [Integrations](/integrations?tab=docker-registry) page
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SecretMount(UniversalBaseModel):
    type: typing.Literal["secret"] = pydantic.Field(default="secret")
    """
    +value=secret
    """

    mount_path: str = pydantic.Field()
    """
    Absolute file path where the file will be created.
    """

    secret_fqn: str = pydantic.Field()
    """
    The TrueFoundry secret whose value will be the file content.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_chart import EventChart
from .event_involved_object import EventInvolvedObject


class Event(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the event
    """

    first_timestamp: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="firstTimestamp")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp when the event was first observed
    """

    last_timestamp: typing_extensions.Annotated[str, FieldMetadata(alias="lastTimestamp")] = pydantic.Field()
    """
    Timestamp when the event was last observed
    """

    involved_object: typing_extensions.Annotated[EventInvolvedObject, FieldMetadata(alias="involvedObject")] = (
        pydantic.Field()
    )
    """
    Details of the involved object
    """

    type: str = pydantic.Field()
    """
    Type of the event
    """

    count: int = pydantic.Field()
    """
    Number of occurrences of the event
    """

    reason: str = pydantic.Field()
    """
    Reason for the event
    """

    message: str = pydantic.Field()
    """
    Message describing the event
    """

    namespace: typing.Optional[str] = pydantic.Field(default=None)
    """
    Namespace of the event
    """

    chart: EventChart = pydantic.Field()
    """
    Chart of the event
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .http_error_code import HttpErrorCode


class HttpError(UniversalBaseModel):
    status_code: typing_extensions.Annotated[int, FieldMetadata(alias="statusCode")] = pydantic.Field()
    """
    HTTP Status Code
    """

    message: str = pydantic.Field()
    """
    Error Message
    """

    code: typing.Optional[HttpErrorCode] = pydantic.Field(default=None)
    """
    Error code
    """

    details: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field(default=None)
    """
    Error details
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SparkBuild(UniversalBaseModel):
    """
    Describes that we are using python to build a container image with a specific python version and pip packages installed.
    """

    type: typing.Literal["tfy-spark-buildpack"] = pydantic.Field(default="tfy-spark-buildpack")
    """
    +value=tfy-spark-buildpack
    """

    spark_version: str = pydantic.Field(default="3.5.2")
    """
    Spark version should match the spark version installed in the image.
    """

    build_context_path: str = pydantic.Field(default="./")
    """
    Build path relative to project root path.
    """

    requirements_path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Path to `requirements.txt` relative to
    `Path to build context`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CronMetric(UniversalBaseModel):
    type: typing.Literal["cron"] = pydantic.Field(default="cron")
    """
    +value=cron
    """

    desired_replicas: typing.Optional[int] = pydantic.Field(default=None)
    """
    Desired number of replicas during the given interval. Default value is max_replicas.
    """

    start: str = pydantic.Field()
    """
    Cron expression indicating the start of the cron schedule.
    ```
    * * * * *
    | | | | |
    | | | | |___ day of week (0-6) (Sunday is 0)
    | | | |_____ month (1-12)
    | | |_______ day of month (1-31)
    | |_________ hour (0-23)
    |___________ minute (0-59)
    ```
    """

    end: str = pydantic.Field()
    """
    Cron expression indicating the end of the cron schedule.
    ```
    * * * * *
    | | | | |
    | | | | |___ day of week (0-6) (Sunday is 0)
    | | | |_____ month (1-12)
    | | |_______ day of month (1-31)
    | |_________ hour (0-23)
    |___________ minute (0-59)
    ```
    """

    timezone: str = pydantic.Field(default="UTC")
    """
    Timezone against which the cron schedule will be calculated, e.g. "Asia/Tokyo". Default is machine's local time.
    https://docs.truefoundry.com/docs/list-of-supported-timezones
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class PortAppProtocol(enum.StrEnum):
    """
    Application Protocol for the port.
    Select the application protocol used by your service. For most use cases, this should be `http`(HTTP/1.1).
    If you are running a gRPC server, select the `grpc` option.
    This is only applicable if `expose=true`.
    """

    HTTP = "http"
    GRPC = "grpc"
    TCP = "tcp"

    def visit(
        self,
        http: typing.Callable[[], T_Result],
        grpc: typing.Callable[[], T_Result],
        tcp: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PortAppProtocol.HTTP:
            return http()
        if self is PortAppProtocol.GRPC:
            return grpc()
        if self is PortAppProtocol.TCP:
            return tcp()



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_ecr import AwsEcr
from .aws_eks_integration import AwsEksIntegration
from .aws_parameter_store import AwsParameterStore
from .aws_s3 import AwsS3
from .aws_secrets_manager import AwsSecretsManager

AwsIntegrations = typing.Union[AwsEcr, AwsS3, AwsParameterStore, AwsSecretsManager, AwsEksIntegration]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TokenPagination(UniversalBaseModel):
    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of items per page
    """

    next_page_token: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="nextPageToken")] = (
        pydantic.Field(default=None)
    )
    """
    Base64 encoded token for the next page
    """

    previous_page_token: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="previousPageToken")] = (
        pydantic.Field(default=None)
    )
    """
    Base64 encoded token for the previous page
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



import os
import typing

import httpx
from .base_client import AsyncBaseTrueFoundry, BaseTrueFoundry


class TrueFoundry(BaseTrueFoundry):
    def __init__(
        self,
        *,
        base_url: str,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("TFY_API_KEY"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        super().__init__(
            base_url=base_url,
            api_key=api_key,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
        )

    @property
    def applications(self):
        if self._applications is None:
            from ._wrapped_clients import WrappedApplicationsClient  # noqa: E402

            self._applications = WrappedApplicationsClient(client_wrapper=self._client_wrapper)
        return self._applications

    @property
    def artifacts(self):
        if self._artifacts is None:
            from ._wrapped_clients import WrappedArtifactsClient  # noqa: E402

            self._artifacts = WrappedArtifactsClient(client_wrapper=self._client_wrapper)
        return self._artifacts

    @property
    def artifact_versions(self):
        if self._artifact_versions is None:
            from ._wrapped_clients import WrappedArtifactVersionsClient  # noqa: E402

            self._artifact_versions = WrappedArtifactVersionsClient(client_wrapper=self._client_wrapper)
        return self._artifact_versions

    @property
    def data_directories(self):
        if self._data_directories is None:
            from ._wrapped_clients import WrappedDataDirectoriesClient  # noqa: E402

            self._data_directories = WrappedDataDirectoriesClient(client_wrapper=self._client_wrapper)
        return self._data_directories

    @property
    def models(self):
        if self._models is None:
            from ._wrapped_clients import WrappedModelsClient  # noqa: E402

            self._models = WrappedModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def model_versions(self):
        if self._model_versions is None:
            from ._wrapped_clients import WrappedModelVersionsClient  # noqa: E402

            self._model_versions = WrappedModelVersionsClient(client_wrapper=self._client_wrapper)
        return self._model_versions

    @property
    def prompts(self):
        if self._prompts is None:
            from ._wrapped_clients import WrappedPromptsClient  # noqa: E402

            self._prompts = WrappedPromptsClient(client_wrapper=self._client_wrapper)
        return self._prompts

    @property
    def prompt_versions(self):
        if self._prompt_versions is None:
            from ._wrapped_clients import WrappedPromptVersionsClient  # noqa: E402

            self._prompt_versions = WrappedPromptVersionsClient(client_wrapper=self._client_wrapper)
        return self._prompt_versions

    @property
    def secret_groups(self):
        if self._secret_groups is None:
            from ._wrapped_clients import WrappedSecretGroupsClient  # noqa: E402

            self._secret_groups = WrappedSecretGroupsClient(client_wrapper=self._client_wrapper)
        return self._secret_groups

    @property
    def workspaces(self):
        if self._workspaces is None:
            from ._wrapped_clients import WrappedWorkspacesClient  # noqa: E402

            self._workspaces = WrappedWorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces


class AsyncTrueFoundry(AsyncBaseTrueFoundry):
    def __init__(
        self,
        *,
        base_url: str,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("TFY_API_KEY"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        super().__init__(
            base_url=base_url,
            api_key=api_key,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
        )

    @property
    def applications(self):
        if self._applications is None:
            from ._wrapped_clients import WrappedAsyncApplicationsClient  # noqa: E402

            self._applications = WrappedAsyncApplicationsClient(client_wrapper=self._client_wrapper)
        return self._applications

    @property
    def artifacts(self):
        if self._artifacts is None:
            from ._wrapped_clients import WrappedAsyncArtifactsClient  # noqa: E402

            self._artifacts = WrappedAsyncArtifactsClient(client_wrapper=self._client_wrapper)
        return self._artifacts

    @property
    def artifact_versions(self):
        if self._artifact_versions is None:
            from ._wrapped_clients import WrappedAsyncArtifactVersionsClient  # noqa: E402

            self._artifact_versions = WrappedAsyncArtifactVersionsClient(client_wrapper=self._client_wrapper)
        return self._artifact_versions

    @property
    def data_directories(self):
        if self._data_directories is None:
            from ._wrapped_clients import WrappedAsyncDataDirectoriesClient  # noqa: E402

            self._data_directories = WrappedAsyncDataDirectoriesClient(client_wrapper=self._client_wrapper)
        return self._data_directories

    @property
    def models(self):
        if self._models is None:
            from ._wrapped_clients import WrappedAsyncModelsClient  # noqa: E402

            self._models = WrappedAsyncModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def model_versions(self):
        if self._model_versions is None:
            from ._wrapped_clients import WrappedAsyncModelVersionsClient  # noqa: E402

            self._model_versions = WrappedAsyncModelVersionsClient(client_wrapper=self._client_wrapper)
        return self._model_versions

    @property
    def prompts(self):
        if self._prompts is None:
            from ._wrapped_clients import WrappedAsyncPromptsClient  # noqa: E402

            self._prompts = WrappedAsyncPromptsClient(client_wrapper=self._client_wrapper)
        return self._prompts

    @property
    def prompt_versions(self):
        if self._prompt_versions is None:
            from ._wrapped_clients import WrappedAsyncPromptVersionsClient  # noqa: E402

            self._prompt_versions = WrappedAsyncPromptVersionsClient(client_wrapper=self._client_wrapper)
        return self._prompt_versions

    @property
    def secret_groups(self):
        if self._secret_groups is None:
            from ._wrapped_clients import WrappedAsyncSecretGroupsClient  # noqa: E402

            self._secret_groups = WrappedAsyncSecretGroupsClient(client_wrapper=self._client_wrapper)
        return self._secret_groups

    @property
    def workspaces(self):
        if self._workspaces is None:
            from ._wrapped_clients import WrappedAsyncWorkspacesClient  # noqa: E402

            self._workspaces = WrappedAsyncWorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces


TrueFoundry.__doc__ = BaseTrueFoundry.__doc__
AsyncTrueFoundry.__doc__ = AsyncBaseTrueFoundry.__doc__



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .aws_region import AwsRegion
from .aws_s3auth_data import AwsS3AuthData


class AwsS3(UniversalBaseModel):
    """
    AWS S3
    """

    type: typing.Literal["integration/blob-storage/aws/s3"] = pydantic.Field(default="integration/blob-storage/aws/s3")
    """
    +value=integration/blob-storage/aws/s3
    """

    name: str = pydantic.Field()
    """
    The name of the integration that will be displayed in the TrueFoundry UI.
    """

    storage_root: str = pydantic.Field()
    """
    The S3 storage path in the format 's3://<bucket-name>/<optional-path-prefix>'. Works with S3-compatible storage providers like MinIO, Ceph, and DigitalOcean Spaces. Example: 's3://mybucket' or 's3://mybucket/artifacts/production'.
    """

    region: AwsRegion
    auth_data: typing.Optional[AwsS3AuthData] = pydantic.Field(default=None)
    """
    Custom authentication data for the integration.
    """

    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of subjects that are authorized to access this integration. List of user fqn in format <user_type>:<username>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class UserRoleWithResourceResourceType(enum.StrEnum):
    ROLE = "role"
    ACCOUNT = "account"
    CLUSTER = "cluster"
    WORKSPACE = "workspace"
    ENVIRONMENT = "environment"
    SECRET_GROUP = "secret-group"
    DEPLOYMENT = "deployment"
    APPLICATION = "application"
    SECRET = "secret"
    DOCKER_REGISTRY = "docker-registry"
    TENANT = "tenant"
    VCS_INTEGRATION = "vcs-integration"
    MLF_PROJECT = "mlf-project"
    PROVIDER_INTEGRATION = "provider-integration"
    TEAM = "team"
    SERVICE_ACCOUNT = "service-account"
    LLM_GATEWAY = "llm-gateway"
    POLICY = "policy"
    SETTINGS = "settings"
    PROVIDER_ACCOUNT = "provider-account"
    USER = "user"
    ALERT_CONFIG = "alert-config"
    ALERT_RULE = "alert-rule"
    GATEWAY_CONFIG = "gateway-config"
    TRACING_PROJECT = "tracing-project"
    TRACING_APPLICATION = "tracing-application"
    AGENT_APP = "agent-app"
    GATEWAY_INSTALLATION = "gateway-installation"
    VIRTUAL_ACCOUNT = "virtual-account"
    SSO_BASED_IDENTITY = "sso-based-identity"

    def visit(
        self,
        role: typing.Callable[[], T_Result],
        account: typing.Callable[[], T_Result],
        cluster: typing.Callable[[], T_Result],
        workspace: typing.Callable[[], T_Result],
        environment: typing.Callable[[], T_Result],
        secret_group: typing.Callable[[], T_Result],
        deployment: typing.Callable[[], T_Result],
        application: typing.Callable[[], T_Result],
        secret: typing.Callable[[], T_Result],
        docker_registry: typing.Callable[[], T_Result],
        tenant: typing.Callable[[], T_Result],
        vcs_integration: typing.Callable[[], T_Result],
        mlf_project: typing.Callable[[], T_Result],
        provider_integration: typing.Callable[[], T_Result],
        team: typing.Callable[[], T_Result],
        service_account: typing.Callable[[], T_Result],
        llm_gateway: typing.Callable[[], T_Result],
        policy: typing.Callable[[], T_Result],
        settings: typing.Callable[[], T_Result],
        provider_account: typing.Callable[[], T_Result],
        user: typing.Callable[[], T_Result],
        alert_config: typing.Callable[[], T_Result],
        alert_rule: typing.Callable[[], T_Result],
        gateway_config: typing.Callable[[], T_Result],
        tracing_project: typing.Callable[[], T_Result],
        tracing_application: typing.Callable[[], T_Result],
        agent_app: typing.Callable[[], T_Result],
        gateway_installation: typing.Callable[[], T_Result],
        virtual_account: typing.Callable[[], T_Result],
        sso_based_identity: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is UserRoleWithResourceResourceType.ROLE:
            return role()
        if self is UserRoleWithResourceResourceType.ACCOUNT:
            return account()
        if self is UserRoleWithResourceResourceType.CLUSTER:
            return cluster()
        if self is UserRoleWithResourceResourceType.WORKSPACE:
            return workspace()
        if self is UserRoleWithResourceResourceType.ENVIRONMENT:
            return environment()
        if self is UserRoleWithResourceResourceType.SECRET_GROUP:
            return secret_group()
        if self is UserRoleWithResourceResourceType.DEPLOYMENT:
            return deployment()
        if self is UserRoleWithResourceResourceType.APPLICATION:
            return application()
        if self is UserRoleWithResourceResourceType.SECRET:
            return secret()
        if self is UserRoleWithResourceResourceType.DOCKER_REGISTRY:
            return docker_registry()
        if self is UserRoleWithResourceResourceType.TENANT:
            return tenant()
        if self is UserRoleWithResourceResourceType.VCS_INTEGRATION:
            return vcs_integration()
        if self is UserRoleWithResourceResourceType.MLF_PROJECT:
            return mlf_project()
        if self is UserRoleWithResourceResourceType.PROVIDER_INTEGRATION:
            return provider_integration()
        if self is UserRoleWithResourceResourceType.TEAM:
            return team()
        if self is UserRoleWithResourceResourceType.SERVICE_ACCOUNT:
            return service_account()
        if self is UserRoleWithResourceResourceType.LLM_GATEWAY:
            return llm_gateway()
        if self is UserRoleWithResourceResourceType.POLICY:
            return policy()
        if self is UserRoleWithResourceResourceType.SETTINGS:
            return settings()
        if self is UserRoleWithResourceResourceType.PROVIDER_ACCOUNT:
            return provider_account()
        if self is UserRoleWithResourceResourceType.USER:
            return user()
        if self is UserRoleWithResourceResourceType.ALERT_CONFIG:
            return alert_config()
        if self is UserRoleWithResourceResourceType.ALERT_RULE:
            return alert_rule()
        if self is UserRoleWithResourceResourceType.GATEWAY_CONFIG:
            return gateway_config()
        if self is UserRoleWithResourceResourceType.TRACING_PROJECT:
            return tracing_project()
        if self is UserRoleWithResourceResourceType.TRACING_APPLICATION:
            return tracing_application()
        if self is UserRoleWithResourceResourceType.AGENT_APP:
            return agent_app()
        if self is UserRoleWithResourceResourceType.GATEWAY_INSTALLATION:
            return gateway_installation()
        if self is UserRoleWithResourceResourceType.VIRTUAL_ACCOUNT:
            return virtual_account()
        if self is UserRoleWithResourceResourceType.SSO_BASED_IDENTITY:
            return sso_based_identity()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .environment_color import EnvironmentColor
from .environment_optimize_for import EnvironmentOptimizeFor


class EnvironmentManifest(UniversalBaseModel):
    type: typing.Literal["environment"] = pydantic.Field(default="environment")
    """
    +value=environment
    """

    name: str = pydantic.Field()
    """
    Environment Name
    """

    color: EnvironmentColor
    is_production: typing_extensions.Annotated[bool, FieldMetadata(alias="isProduction")] = pydantic.Field()
    """
    Environment Type - Indicates if the environment is for production use
    """

    optimize_for: typing_extensions.Annotated[EnvironmentOptimizeFor, FieldMetadata(alias="optimizeFor")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .jwt_auth_config_claims_item import JwtAuthConfigClaimsItem


class JwtAuthConfig(UniversalBaseModel):
    """
    Configure JWT-based authentication using JWKS
    """

    type: typing.Literal["jwt_auth"] = pydantic.Field(default="jwt_auth")
    """
    +value=jwt_auth
    """

    integration_fqn: str = pydantic.Field()
    """
    FQN of the JWT Auth integration. If you can't find your integration here,
    add it through the [Integrations](/integrations) page
    """

    enable_login: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable login for the service
    """

    claims: typing.Optional[typing.List[JwtAuthConfigClaimsItem]] = pydantic.Field(default=None)
    """
    List of key-value pairs of claims to verify in the JWT token
    """

    bypass_auth_paths: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of paths that will bypass auth.
    needs to start with a forward slash(/) and should not contain wildcards(*)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class BuildStatus(enum.StrEnum):
    STARTED = "STARTED"
    SUCCEEDED = "SUCCEEDED"
    FAILED = "FAILED"

    def visit(
        self,
        started: typing.Callable[[], T_Result],
        succeeded: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is BuildStatus.STARTED:
            return started()
        if self is BuildStatus.SUCCEEDED:
            return succeeded()
        if self is BuildStatus.FAILED:
            return failed()



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.application_type import ApplicationType
from ...types.deployment_build import DeploymentBuild
from ...types.deployment_status import DeploymentStatus
from ...types.get_suggested_deployment_endpoint_response import GetSuggestedDeploymentEndpointResponse
from ...types.presigned_url_object import PresignedUrlObject
from .raw_client import AsyncRawDeploymentsClient, RawDeploymentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DeploymentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDeploymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDeploymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDeploymentsClient
        """
        return self._raw_client

    def get_deployment_statuses(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeploymentStatus]:
        """
        This endpoint returns all statuses for a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeploymentStatus]
            Deployment statuses returned successfully.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.deployments.get_deployment_statuses(
            id="id",
            deployment_id="deploymentId",
        )
        """
        _response = self._raw_client.get_deployment_statuses(id, deployment_id, request_options=request_options)
        return _response.data

    def get_builds(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeploymentBuild]:
        """
        This endpoint returns all build details associated with a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeploymentBuild]
            Deployment builds returned successfully.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.deployments.get_builds(
            id="id",
            deployment_id="deploymentId",
        )
        """
        _response = self._raw_client.get_builds(id, deployment_id, request_options=request_options)
        return _response.data

    def get_code_upload_url(
        self, *, service_name: str, workspace_fqn: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PresignedUrlObject:
        """
        Generate presigned URL to upload code for given serviceName and workspaceFqn

        Parameters
        ----------
        service_name : str
            ServiceName of the deployment

        workspace_fqn : str
            WorkSpaceFQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PresignedUrlObject
            Returns a presigned URL and URI as json object

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.deployments.get_code_upload_url(
            service_name="serviceName",
            workspace_fqn="workspaceFqn",
        )
        """
        _response = self._raw_client.get_code_upload_url(
            service_name=service_name, workspace_fqn=workspace_fqn, request_options=request_options
        )
        return _response.data

    def get_suggested_endpoint(
        self,
        *,
        application_type: ApplicationType,
        application_name: str,
        workspace_id: str,
        base_domain: typing.Optional[str] = None,
        port: typing.Optional[str] = None,
        prefer_wildcard: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSuggestedDeploymentEndpointResponse:
        """
        Generate deployment endpoint based on the provided query parameters.

        Parameters
        ----------
        application_type : ApplicationType
            Application Type

        application_name : str
            Application name

        workspace_id : str
            Workspace id

        base_domain : typing.Optional[str]
            Base domain

        port : typing.Optional[str]
            Port

        prefer_wildcard : typing.Optional[bool]
            Prefer wildcard

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSuggestedDeploymentEndpointResponse
            Returns host and path as json object

        Examples
        --------
        from truefoundry_sdk import ApplicationType, TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.deployments.get_suggested_endpoint(
            application_type=ApplicationType.ASYNC_SERVICE,
            application_name="applicationName",
            workspace_id="workspaceId",
            base_domain="baseDomain",
            port="port",
            prefer_wildcard=True,
        )
        """
        _response = self._raw_client.get_suggested_endpoint(
            application_type=application_type,
            application_name=application_name,
            workspace_id=workspace_id,
            base_domain=base_domain,
            port=port,
            prefer_wildcard=prefer_wildcard,
            request_options=request_options,
        )
        return _response.data


class AsyncDeploymentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDeploymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDeploymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDeploymentsClient
        """
        return self._raw_client

    async def get_deployment_statuses(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeploymentStatus]:
        """
        This endpoint returns all statuses for a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeploymentStatus]
            Deployment statuses returned successfully.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.deployments.get_deployment_statuses(
                id="id",
                deployment_id="deploymentId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_deployment_statuses(id, deployment_id, request_options=request_options)
        return _response.data

    async def get_builds(
        self, id: str, deployment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeploymentBuild]:
        """
        This endpoint returns all build details associated with a specific deployment in a given application.

        Parameters
        ----------
        id : str
            Application id of the application

        deployment_id : str
            Deployment id of the deployment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeploymentBuild]
            Deployment builds returned successfully.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.deployments.get_builds(
                id="id",
                deployment_id="deploymentId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_builds(id, deployment_id, request_options=request_options)
        return _response.data

    async def get_code_upload_url(
        self, *, service_name: str, workspace_fqn: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PresignedUrlObject:
        """
        Generate presigned URL to upload code for given serviceName and workspaceFqn

        Parameters
        ----------
        service_name : str
            ServiceName of the deployment

        workspace_fqn : str
            WorkSpaceFQN of the workspace

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PresignedUrlObject
            Returns a presigned URL and URI as json object

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.deployments.get_code_upload_url(
                service_name="serviceName",
                workspace_fqn="workspaceFqn",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_code_upload_url(
            service_name=service_name, workspace_fqn=workspace_fqn, request_options=request_options
        )
        return _response.data

    async def get_suggested_endpoint(
        self,
        *,
        application_type: ApplicationType,
        application_name: str,
        workspace_id: str,
        base_domain: typing.Optional[str] = None,
        port: typing.Optional[str] = None,
        prefer_wildcard: typing.Optional[bool] = False,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSuggestedDeploymentEndpointResponse:
        """
        Generate deployment endpoint based on the provided query parameters.

        Parameters
        ----------
        application_type : ApplicationType
            Application Type

        application_name : str
            Application name

        workspace_id : str
            Workspace id

        base_domain : typing.Optional[str]
            Base domain

        port : typing.Optional[str]
            Port

        prefer_wildcard : typing.Optional[bool]
            Prefer wildcard

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSuggestedDeploymentEndpointResponse
            Returns host and path as json object

        Examples
        --------
        import asyncio

        from truefoundry_sdk import ApplicationType, AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.deployments.get_suggested_endpoint(
                application_type=ApplicationType.ASYNC_SERVICE,
                application_name="applicationName",
                workspace_id="workspaceId",
                base_domain="baseDomain",
                port="port",
                prefer_wildcard=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_suggested_endpoint(
            application_type=application_type,
            application_name=application_name,
            workspace_id=workspace_id,
            base_domain=base_domain,
            port=port,
            prefer_wildcard=prefer_wildcard,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.artifact import Artifact
from ..types.artifact_manifest import ArtifactManifest
from ..types.empty_response import EmptyResponse
from ..types.get_artifact_response import GetArtifactResponse
from ..types.get_artifact_version_response import GetArtifactVersionResponse
from ..types.list_artifacts_response import ListArtifactsResponse
from .raw_client import AsyncRawArtifactsClient, RawArtifactsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ArtifactsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawArtifactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawArtifactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawArtifactsClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetArtifactResponse:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.artifacts.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.artifacts.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def list(
        self,
        *,
        fqn: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        run_id: typing.Optional[str] = None,
        include_empty_artifacts: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Artifact, ListArtifactsResponse]:
        """
        Parameters
        ----------
        fqn : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        run_id : typing.Optional[str]

        include_empty_artifacts : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Artifact, ListArtifactsResponse]
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.artifacts.list(
            fqn="fqn",
            ml_repo_id="ml_repo_id",
            name="name",
            offset=1,
            limit=1,
            run_id="run_id",
            include_empty_artifacts=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            fqn=fqn,
            ml_repo_id=ml_repo_id,
            name=name,
            offset=offset,
            limit=limit,
            run_id=run_id,
            include_empty_artifacts=include_empty_artifacts,
            request_options=request_options,
        )

    def create_or_update(
        self, *, manifest: ArtifactManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetArtifactVersionResponse:
        """
        Parameters
        ----------
        manifest : ArtifactManifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactVersionResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import (
            ArtifactManifest,
            TrueFoundry,
            TrueFoundryManagedSource,
        )

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.artifacts.create_or_update(
            manifest=ArtifactManifest(
                name="name",
                metadata={"key": "value"},
                ml_repo="ml_repo",
                source=TrueFoundryManagedSource(),
            ),
        )
        """
        _response = self._raw_client.create_or_update(manifest=manifest, request_options=request_options)
        return _response.data


class AsyncArtifactsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawArtifactsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawArtifactsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawArtifactsClient
        """
        return self._raw_client

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetArtifactResponse:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.artifacts.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.artifacts.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def list(
        self,
        *,
        fqn: typing.Optional[str] = None,
        ml_repo_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        offset: typing.Optional[int] = 0,
        limit: typing.Optional[int] = 100,
        run_id: typing.Optional[str] = None,
        include_empty_artifacts: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Artifact, ListArtifactsResponse]:
        """
        Parameters
        ----------
        fqn : typing.Optional[str]

        ml_repo_id : typing.Optional[str]

        name : typing.Optional[str]

        offset : typing.Optional[int]

        limit : typing.Optional[int]

        run_id : typing.Optional[str]

        include_empty_artifacts : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Artifact, ListArtifactsResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.artifacts.list(
                fqn="fqn",
                ml_repo_id="ml_repo_id",
                name="name",
                offset=1,
                limit=1,
                run_id="run_id",
                include_empty_artifacts=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            fqn=fqn,
            ml_repo_id=ml_repo_id,
            name=name,
            offset=offset,
            limit=limit,
            run_id=run_id,
            include_empty_artifacts=include_empty_artifacts,
            request_options=request_options,
        )

    async def create_or_update(
        self, *, manifest: ArtifactManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetArtifactVersionResponse:
        """
        Parameters
        ----------
        manifest : ArtifactManifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetArtifactVersionResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import (
            ArtifactManifest,
            AsyncTrueFoundry,
            TrueFoundryManagedSource,
        )

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.artifacts.create_or_update(
                manifest=ArtifactManifest(
                    name="name",
                    metadata={"key": "value"},
                    ml_repo="ml_repo",
                    source=TrueFoundryManagedSource(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(manifest=manifest, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...types.logs_filter_query import LogsFilterQuery
from ...types.logs_response import LogsResponse


class RawBuildLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        pipeline_run_name: str,
        *,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        direction: typing.Optional[str] = None,
        filter_query: typing.Optional[LogsFilterQuery] = None,
        num_logs_to_ignore: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LogsResponse]:
        """
        Get logs for a given pipeline run by its name, with optional filters and time range.

        Parameters
        ----------
        pipeline_run_name : str
            PipelineRun Name

        start_ts : typing.Optional[str]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[str]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[str]
            Max number of log lines to fetch

        direction : typing.Optional[str]
            Direction of sorting logs. Can be `asc` or `desc`

        filter_query : typing.Optional[LogsFilterQuery]
            Query to filter logs

        num_logs_to_ignore : typing.Optional[float]
            Number of logs corresponding to the starting timestamp to be ignored.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LogsResponse]
            Successfully retrieved build logs for the pipeline run
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/build-logs/{jsonable_encoder(pipeline_run_name)}",
            method="GET",
            params={
                "startTs": start_ts,
                "endTs": end_ts,
                "limit": limit,
                "direction": direction,
                "filterQuery": convert_and_respect_annotation_metadata(
                    object_=filter_query, annotation=LogsFilterQuery, direction="write"
                ),
                "numLogsToIgnore": num_logs_to_ignore,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogsResponse,
                    parse_obj_as(
                        type_=LogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBuildLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        pipeline_run_name: str,
        *,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        direction: typing.Optional[str] = None,
        filter_query: typing.Optional[LogsFilterQuery] = None,
        num_logs_to_ignore: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LogsResponse]:
        """
        Get logs for a given pipeline run by its name, with optional filters and time range.

        Parameters
        ----------
        pipeline_run_name : str
            PipelineRun Name

        start_ts : typing.Optional[str]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[str]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[str]
            Max number of log lines to fetch

        direction : typing.Optional[str]
            Direction of sorting logs. Can be `asc` or `desc`

        filter_query : typing.Optional[LogsFilterQuery]
            Query to filter logs

        num_logs_to_ignore : typing.Optional[float]
            Number of logs corresponding to the starting timestamp to be ignored.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LogsResponse]
            Successfully retrieved build logs for the pipeline run
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/build-logs/{jsonable_encoder(pipeline_run_name)}",
            method="GET",
            params={
                "startTs": start_ts,
                "endTs": end_ts,
                "limit": limit,
                "direction": direction,
                "filterQuery": convert_and_respect_annotation_metadata(
                    object_=filter_query, annotation=LogsFilterQuery, direction="write"
                ),
                "numLogsToIgnore": num_logs_to_ignore,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogsResponse,
                    parse_obj_as(
                        type_=LogsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class MirrorAction(UniversalBaseModel):
    type: typing.Literal["mirror"] = pydantic.Field(default="mirror")
    """
    +value=mirror
    """

    service_uri: str = pydantic.Field()
    """
    Service uri to mirror to. This is the kubernetes fully qualified domain name that the target service can be reached on
    Eg - For a service called `sample-service` in workspace `ws` this would be `sample-service.ws.svc.cluster.local
    """

    port: int = pydantic.Field()
    """
    Port to redirect the service traffic to
    """

    percentage: int = pydantic.Field()
    """
    Percentage of requests to mirror
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SparkExecutorFixedInstances(UniversalBaseModel):
    """
    Fixed Instances
    """

    type: typing.Literal["fixed"] = pydantic.Field(default="fixed")
    """
    +value=fixed
    """

    count: int = pydantic.Field(default=1)
    """
    Number of instances to start
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class RolePermissions(UniversalBaseModel):
    resource_type: typing_extensions.Annotated[str, FieldMetadata(alias="resourceType")] = pydantic.Field()
    """
    Type of resource this role can perform actions on
    """

    actions: typing.List[str] = pydantic.Field()
    """
    Actions this role can perform on the resource
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .owned_by import OwnedBy


class PersonalAccessTokenManifest(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Personal Access Token Name
    """

    type: typing.Literal["pat"] = pydantic.Field(default="pat")
    """
    +value=pat
    """

    expiration_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    Expiration Date of the Personal Access Token (should be in the ISO 8601 format)
    """

    user_fqn: str = pydantic.Field()
    """
    The fully qualified name of the user
    """

    owned_by: typing_extensions.Annotated[typing.Optional[OwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .pangea_guard_type import PangeaGuardType
from .pangea_guardrail_config_operation import PangeaGuardrailConfigOperation
from .pangea_key_auth import PangeaKeyAuth


class PangeaGuardrailConfig(UniversalBaseModel):
    """
    Pangea
    """

    name: str = pydantic.Field()
    """
    The name of the Guardrail Config.
    """

    type: typing.Literal["integration/guardrail-config/pangea"] = pydantic.Field(
        default="integration/guardrail-config/pangea"
    )
    """
    +uiType=Hidden
    +value=integration/guardrail-config/pangea
    """

    auth_data: PangeaKeyAuth
    guard_type: PangeaGuardType
    operation: typing.Optional[PangeaGuardrailConfigOperation] = pydantic.Field(default=None)
    """
    The operation type to use for the Guardrail. Validate guardrails are used to validate requests and mutate can validate as well as mutate requests.
    """

    domain: str = pydantic.Field()
    """
    Domain of the cloud provider and region where your Pangea project is configured. Example: if endpoint is: https://<service_name>.aws.us-west-2.pangea.cloud/v1/text/guard, the input should be: aws.us-west-2.pangea.cloud
    """

    recipe: typing.Optional[str] = pydantic.Field(default=None)
    """
    Recipe key from Pangea console defining security rules and data types to apply
    """

    overrides: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Overrides for the Pangea account
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class QuayBasicAuth(UniversalBaseModel):
    """
    Quay Basic Auth
    """

    type: typing.Literal["basic-auth"] = pydantic.Field(default="basic-auth")
    """
    +value=basic-auth
    """

    username: str = pydantic.Field()
    """
    The username for the basic authentication.
    """

    password: str = pydantic.Field()
    """
    The password for the basic authentication.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_logs_response import GetLogsResponse
from ..types.logs_search_filter_type import LogsSearchFilterType
from ..types.logs_search_operator_type import LogsSearchOperatorType
from ..types.logs_sorting_direction import LogsSortingDirection
from .raw_client import AsyncRawLogsClient, RawLogsClient


class LogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLogsClient
        """
        return self._raw_client

    def get(
        self,
        *,
        start_ts: typing.Optional[int] = None,
        end_ts: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        direction: typing.Optional[LogsSortingDirection] = None,
        num_logs_to_ignore: typing.Optional[int] = None,
        application_id: typing.Optional[str] = None,
        application_fqn: typing.Optional[str] = None,
        deployment_id: typing.Optional[str] = None,
        job_run_name: typing.Optional[str] = None,
        pod_name: typing.Optional[str] = None,
        container_name: typing.Optional[str] = None,
        pod_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pod_names_regex: typing.Optional[str] = None,
        search_filters: typing.Optional[str] = None,
        search_string: typing.Optional[str] = None,
        search_type: typing.Optional[LogsSearchFilterType] = None,
        search_operator: typing.Optional[LogsSearchOperatorType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetLogsResponse:
        """
        Fetch logs for various workload components, including Services, Jobs, Workflows, Job Runs, and Pods. Logs are filtered based on the provided query parameters.

        Parameters
        ----------
        start_ts : typing.Optional[int]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[int]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[int]
            Max number of log lines to fetch

        direction : typing.Optional[LogsSortingDirection]
            Direction of sorting logs. Can be `asc` or `desc`

        num_logs_to_ignore : typing.Optional[int]
            Number of logs corresponding to the starting timestamp to be ignored.

        application_id : typing.Optional[str]
            Application ID

        application_fqn : typing.Optional[str]
            Application FQN

        deployment_id : typing.Optional[str]
            Deployment ID

        job_run_name : typing.Optional[str]
            Name of the Job Run for which to fetch logs.

        pod_name : typing.Optional[str]
            Name of Pod for which to fetch logs.

        container_name : typing.Optional[str]
            Name of the Container for which to fetch logs.

        pod_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of pod names for which to fetch logs.

        pod_names_regex : typing.Optional[str]
            Regex pattern for pod names to fetch logs.

        search_filters : typing.Optional[str]
            JSON string containing array of search filters with string, type and operator

        search_string : typing.Optional[str]
            String that needs to be matched

        search_type : typing.Optional[LogsSearchFilterType]
            Query filter type, `regex` `substring` `ignore_case_substring`

        search_operator : typing.Optional[LogsSearchOperatorType]
            Comparison operator for filter. `equal` or `not_equal`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLogsResponse
            Successfully retrieved logs for the workload

        Examples
        --------
        from truefoundry_sdk import (
            LogsSearchFilterType,
            LogsSearchOperatorType,
            LogsSortingDirection,
            TrueFoundry,
        )

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.logs.get(
            start_ts=1000000,
            end_ts=1000000,
            limit=1,
            direction=LogsSortingDirection.ASC,
            num_logs_to_ignore=1,
            application_id="applicationId",
            application_fqn="applicationFqn",
            deployment_id="deploymentId",
            job_run_name="jobRunName",
            pod_name="podName",
            container_name="containerName",
            pod_names_regex="podNamesRegex",
            search_filters="searchFilters",
            search_string="searchString",
            search_type=LogsSearchFilterType.REGEX,
            search_operator=LogsSearchOperatorType.EQUAL,
        )
        """
        _response = self._raw_client.get(
            start_ts=start_ts,
            end_ts=end_ts,
            limit=limit,
            direction=direction,
            num_logs_to_ignore=num_logs_to_ignore,
            application_id=application_id,
            application_fqn=application_fqn,
            deployment_id=deployment_id,
            job_run_name=job_run_name,
            pod_name=pod_name,
            container_name=container_name,
            pod_names=pod_names,
            pod_names_regex=pod_names_regex,
            search_filters=search_filters,
            search_string=search_string,
            search_type=search_type,
            search_operator=search_operator,
            request_options=request_options,
        )
        return _response.data


class AsyncLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLogsClient
        """
        return self._raw_client

    async def get(
        self,
        *,
        start_ts: typing.Optional[int] = None,
        end_ts: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        direction: typing.Optional[LogsSortingDirection] = None,
        num_logs_to_ignore: typing.Optional[int] = None,
        application_id: typing.Optional[str] = None,
        application_fqn: typing.Optional[str] = None,
        deployment_id: typing.Optional[str] = None,
        job_run_name: typing.Optional[str] = None,
        pod_name: typing.Optional[str] = None,
        container_name: typing.Optional[str] = None,
        pod_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        pod_names_regex: typing.Optional[str] = None,
        search_filters: typing.Optional[str] = None,
        search_string: typing.Optional[str] = None,
        search_type: typing.Optional[LogsSearchFilterType] = None,
        search_operator: typing.Optional[LogsSearchOperatorType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetLogsResponse:
        """
        Fetch logs for various workload components, including Services, Jobs, Workflows, Job Runs, and Pods. Logs are filtered based on the provided query parameters.

        Parameters
        ----------
        start_ts : typing.Optional[int]
            Start timestamp for querying logs, in nanoseconds from the Unix epoch.

        end_ts : typing.Optional[int]
            End timestamp for querying logs, in nanoseconds from the Unix epoch.

        limit : typing.Optional[int]
            Max number of log lines to fetch

        direction : typing.Optional[LogsSortingDirection]
            Direction of sorting logs. Can be `asc` or `desc`

        num_logs_to_ignore : typing.Optional[int]
            Number of logs corresponding to the starting timestamp to be ignored.

        application_id : typing.Optional[str]
            Application ID

        application_fqn : typing.Optional[str]
            Application FQN

        deployment_id : typing.Optional[str]
            Deployment ID

        job_run_name : typing.Optional[str]
            Name of the Job Run for which to fetch logs.

        pod_name : typing.Optional[str]
            Name of Pod for which to fetch logs.

        container_name : typing.Optional[str]
            Name of the Container for which to fetch logs.

        pod_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            List of pod names for which to fetch logs.

        pod_names_regex : typing.Optional[str]
            Regex pattern for pod names to fetch logs.

        search_filters : typing.Optional[str]
            JSON string containing array of search filters with string, type and operator

        search_string : typing.Optional[str]
            String that needs to be matched

        search_type : typing.Optional[LogsSearchFilterType]
            Query filter type, `regex` `substring` `ignore_case_substring`

        search_operator : typing.Optional[LogsSearchOperatorType]
            Comparison operator for filter. `equal` or `not_equal`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLogsResponse
            Successfully retrieved logs for the workload

        Examples
        --------
        import asyncio

        from truefoundry_sdk import (
            AsyncTrueFoundry,
            LogsSearchFilterType,
            LogsSearchOperatorType,
            LogsSortingDirection,
        )

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.logs.get(
                start_ts=1000000,
                end_ts=1000000,
                limit=1,
                direction=LogsSortingDirection.ASC,
                num_logs_to_ignore=1,
                application_id="applicationId",
                application_fqn="applicationFqn",
                deployment_id="deploymentId",
                job_run_name="jobRunName",
                pod_name="podName",
                container_name="containerName",
                pod_names_regex="podNamesRegex",
                search_filters="searchFilters",
                search_string="searchString",
                search_type=LogsSearchFilterType.REGEX,
                search_operator=LogsSearchOperatorType.EQUAL,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            start_ts=start_ts,
            end_ts=end_ts,
            limit=limit,
            direction=direction,
            num_logs_to_ignore=num_logs_to_ignore,
            application_id=application_id,
            application_fqn=application_fqn,
            deployment_id=deployment_id,
            job_run_name=job_run_name,
            pod_name=pod_name,
            container_name=container_name,
            pod_names=pod_names,
            pod_names_regex=pod_names_regex,
            search_filters=search_filters,
            search_string=search_string,
            search_type=search_type,
            search_operator=search_operator,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class NvidiaMiggpuProfile(enum.StrEnum):
    """
    Name of the MIG profile to use. One of the following based on gpu type
    Please refer to https://docs.nvidia.com/datacenter/tesla/mig-user-guide/#supported-mig-profiles for more details
    A30 - [1g.6gb, 2g.12gb, 4g.24gb]
    A100 40 GB - [1g.5gb, 1g.10gb, 2g.10gb, 3g.20gb, 4g.20gb, 7g.40gb]
    A100 80 GB / H100 80 GB - [1g.10gb, 1g.20gb, 2g.20gb, 3g.40gb, 4g.40gb, 7g.80gb]
    H100 94 GB - [1g.12gb, 1g.24gb, 2g.24gb, 3g.47gb, 4g.47gb, 7g.94gb]
    H100 96 GB - [1g.12gb, 1g.24gb, 2g.24gb, 3g.48gb, 4g.48gb, 7g.96gb]
    H200 141 GB - [1g.18gb, 1g.35gb, 2g.35gb, 3g.71gb, 4g.71gb]
    B200 180 GB - [1g.23gb, 1g.45gb, 2g.45gb, 3g.90gb, 4g.90gb, 7g.180gb]
    """

    ONE_G6GB = "1g.6gb"
    TWO_G12GB = "2g.12gb"
    ONE_G5GB = "1g.5gb"
    ONE_G10GB = "1g.10gb"
    TWO_G10GB = "2g.10gb"
    THREE_G20GB = "3g.20gb"
    FOUR_G20GB = "4g.20gb"
    ONE_G20GB = "1g.20gb"
    TWO_G20GB = "2g.20gb"
    THREE_G40GB = "3g.40gb"
    FOUR_G40GB = "4g.40gb"
    ONE_G12GB = "1g.12gb"
    ONE_G24GB = "1g.24gb"
    TWO_G24GB = "2g.24gb"
    THREE_G47GB = "3g.47gb"
    FOUR_G47GB = "4g.47gb"
    THREE_G48GB = "3g.48gb"
    FOUR_G48GB = "4g.48gb"
    ONE_G18GB = "1g.18gb"
    ONE_G35GB = "1g.35gb"
    TWO_G35GB = "2g.35gb"
    THREE_G71GB = "3g.71gb"
    FOUR_G71GB = "4g.71gb"
    ONE_G23GB = "1g.23gb"
    ONE_G45GB = "1g.45gb"
    TWO_G45GB = "2g.45gb"
    THREE_G90GB = "3g.90gb"
    FOUR_G90GB = "4g.90gb"
    FOUR_G24GB = "4g.24gb"
    SEVEN_G40GB = "7g.40gb"
    SEVEN_G80GB = "7g.80gb"
    SEVEN_G94GB = "7g.94gb"
    SEVEN_G96GB = "7g.96gb"
    SEVEN_G180GB = "7g.180gb"

    def visit(
        self,
        one_g6gb: typing.Callable[[], T_Result],
        two_g12gb: typing.Callable[[], T_Result],
        one_g5gb: typing.Callable[[], T_Result],
        one_g10gb: typing.Callable[[], T_Result],
        two_g10gb: typing.Callable[[], T_Result],
        three_g20gb: typing.Callable[[], T_Result],
        four_g20gb: typing.Callable[[], T_Result],
        one_g20gb: typing.Callable[[], T_Result],
        two_g20gb: typing.Callable[[], T_Result],
        three_g40gb: typing.Callable[[], T_Result],
        four_g40gb: typing.Callable[[], T_Result],
        one_g12gb: typing.Callable[[], T_Result],
        one_g24gb: typing.Callable[[], T_Result],
        two_g24gb: typing.Callable[[], T_Result],
        three_g47gb: typing.Callable[[], T_Result],
        four_g47gb: typing.Callable[[], T_Result],
        three_g48gb: typing.Callable[[], T_Result],
        four_g48gb: typing.Callable[[], T_Result],
        one_g18gb: typing.Callable[[], T_Result],
        one_g35gb: typing.Callable[[], T_Result],
        two_g35gb: typing.Callable[[], T_Result],
        three_g71gb: typing.Callable[[], T_Result],
        four_g71gb: typing.Callable[[], T_Result],
        one_g23gb: typing.Callable[[], T_Result],
        one_g45gb: typing.Callable[[], T_Result],
        two_g45gb: typing.Callable[[], T_Result],
        three_g90gb: typing.Callable[[], T_Result],
        four_g90gb: typing.Callable[[], T_Result],
        four_g24gb: typing.Callable[[], T_Result],
        seven_g40gb: typing.Callable[[], T_Result],
        seven_g80gb: typing.Callable[[], T_Result],
        seven_g94gb: typing.Callable[[], T_Result],
        seven_g96gb: typing.Callable[[], T_Result],
        seven_g180gb: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is NvidiaMiggpuProfile.ONE_G6GB:
            return one_g6gb()
        if self is NvidiaMiggpuProfile.TWO_G12GB:
            return two_g12gb()
        if self is NvidiaMiggpuProfile.ONE_G5GB:
            return one_g5gb()
        if self is NvidiaMiggpuProfile.ONE_G10GB:
            return one_g10gb()
        if self is NvidiaMiggpuProfile.TWO_G10GB:
            return two_g10gb()
        if self is NvidiaMiggpuProfile.THREE_G20GB:
            return three_g20gb()
        if self is NvidiaMiggpuProfile.FOUR_G20GB:
            return four_g20gb()
        if self is NvidiaMiggpuProfile.ONE_G20GB:
            return one_g20gb()
        if self is NvidiaMiggpuProfile.TWO_G20GB:
            return two_g20gb()
        if self is NvidiaMiggpuProfile.THREE_G40GB:
            return three_g40gb()
        if self is NvidiaMiggpuProfile.FOUR_G40GB:
            return four_g40gb()
        if self is NvidiaMiggpuProfile.ONE_G12GB:
            return one_g12gb()
        if self is NvidiaMiggpuProfile.ONE_G24GB:
            return one_g24gb()
        if self is NvidiaMiggpuProfile.TWO_G24GB:
            return two_g24gb()
        if self is NvidiaMiggpuProfile.THREE_G47GB:
            return three_g47gb()
        if self is NvidiaMiggpuProfile.FOUR_G47GB:
            return four_g47gb()
        if self is NvidiaMiggpuProfile.THREE_G48GB:
            return three_g48gb()
        if self is NvidiaMiggpuProfile.FOUR_G48GB:
            return four_g48gb()
        if self is NvidiaMiggpuProfile.ONE_G18GB:
            return one_g18gb()
        if self is NvidiaMiggpuProfile.ONE_G35GB:
            return one_g35gb()
        if self is NvidiaMiggpuProfile.TWO_G35GB:
            return two_g35gb()
        if self is NvidiaMiggpuProfile.THREE_G71GB:
            return three_g71gb()
        if self is NvidiaMiggpuProfile.FOUR_G71GB:
            return four_g71gb()
        if self is NvidiaMiggpuProfile.ONE_G23GB:
            return one_g23gb()
        if self is NvidiaMiggpuProfile.ONE_G45GB:
            return one_g45gb()
        if self is NvidiaMiggpuProfile.TWO_G45GB:
            return two_g45gb()
        if self is NvidiaMiggpuProfile.THREE_G90GB:
            return three_g90gb()
        if self is NvidiaMiggpuProfile.FOUR_G90GB:
            return four_g90gb()
        if self is NvidiaMiggpuProfile.FOUR_G24GB:
            return four_g24gb()
        if self is NvidiaMiggpuProfile.SEVEN_G40GB:
            return seven_g40gb()
        if self is NvidiaMiggpuProfile.SEVEN_G80GB:
            return seven_g80gb()
        if self is NvidiaMiggpuProfile.SEVEN_G94GB:
            return seven_g94gb()
        if self is NvidiaMiggpuProfile.SEVEN_G96GB:
            return seven_g96gb()
        if self is NvidiaMiggpuProfile.SEVEN_G180GB:
            return seven_g180gb()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .owned_by import OwnedBy
from .true_foundry_integrations import TrueFoundryIntegrations


class TrueFoundryProviderAccount(UniversalBaseModel):
    """
    TrueFoundry
    """

    type: typing.Literal["provider-account/truefoundry"] = pydantic.Field(default="provider-account/truefoundry")
    """
    +value=provider-account/truefoundry
    """

    name: str = pydantic.Field()
    """
    The name of the provider account.
    """

    integrations: typing.Optional[typing.List[TrueFoundryIntegrations]] = pydantic.Field(default=None)
    """
    List of integrations that are associated with the provider account.
    """

    owned_by: typing_extensions.Annotated[typing.Optional[OwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .job_run_status import JobRunStatus
from .subject import Subject


class JobRun(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    JobRun Name
    """

    application_name: typing_extensions.Annotated[str, FieldMetadata(alias="applicationName")] = pydantic.Field()
    """
    Application Name
    """

    deployment_version: typing_extensions.Annotated[str, FieldMetadata(alias="deploymentVersion")] = pydantic.Field()
    """
    Deployment Version
    """

    created_at: typing_extensions.Annotated[float, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    Created At
    """

    end_time: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="endTime")] = pydantic.Field(
        default=None
    )
    """
    End Time of JobRun
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Duration of JobRun
    """

    command: str = pydantic.Field()
    """
    Command
    """

    total_retries: typing_extensions.Annotated[int, FieldMetadata(alias="totalRetries")] = pydantic.Field()
    """
    Total Retries
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error
    """

    status: JobRunStatus = pydantic.Field()
    """
    Status of JobRun
    """

    triggered_by: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="triggeredBy")] = (
        pydantic.Field(default=None)
    )
    """
    Triggered By
    """

    triggered_by_subject: typing_extensions.Annotated[
        typing.Optional[Subject], FieldMetadata(alias="triggeredBySubject")
    ] = pydantic.Field(default=None)
    """
    Triggered By Subject
    """

    exit_code: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="exitCode")] = pydantic.Field(
        default=None
    )
    """
    Exit Code
    """

    spark_ui: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="sparkUi")] = pydantic.Field(
        default=None
    )
    """
    Spark UI Url
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class XgBoostSerializationFormat(enum.StrEnum):
    """
    Serialization format used to save the xgboost model
    """

    CLOUDPICKLE = "cloudpickle"
    JOBLIB = "joblib"
    PICKLE = "pickle"
    JSON = "json"

    def visit(
        self,
        cloudpickle: typing.Callable[[], T_Result],
        joblib: typing.Callable[[], T_Result],
        pickle: typing.Callable[[], T_Result],
        json: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is XgBoostSerializationFormat.CLOUDPICKLE:
            return cloudpickle()
        if self is XgBoostSerializationFormat.JOBLIB:
            return joblib()
        if self is XgBoostSerializationFormat.PICKLE:
            return pickle()
        if self is XgBoostSerializationFormat.JSON:
            return json()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class PangeaGuardType(enum.StrEnum):
    TEXT_GUARD = "textGuard"
    PII = "pii"

    def visit(self, text_guard: typing.Callable[[], T_Result], pii: typing.Callable[[], T_Result]) -> T_Result:
        if self is PangeaGuardType.TEXT_GUARD:
            return text_guard()
        if self is PangeaGuardType.PII:
            return pii()



# This file was auto-generated by Fern from our API Definition.

import typing

from .mcp_server_integration import McpServerIntegration
from .virtual_mcp_server_integration import VirtualMcpServerIntegration

McpServerIntegrations = typing.Union[McpServerIntegration, VirtualMcpServerIntegration]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .owned_by import OwnedBy
from .quay_basic_auth import QuayBasicAuth
from .quay_integrations import QuayIntegrations


class QuayProviderAccount(UniversalBaseModel):
    """
    Quay
    """

    type: typing.Literal["provider-account/quay"] = pydantic.Field(default="provider-account/quay")
    """
    +value=provider-account/quay
    """

    name: str = pydantic.Field()
    """
    The name of the provider account.
    """

    account_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the account on Quay.
    """

    auth_data: typing.Optional[QuayBasicAuth] = None
    integrations: typing.List[QuayIntegrations] = pydantic.Field()
    """
    List of integrations that are associated with the provider account.
    """

    owned_by: typing_extensions.Annotated[typing.Optional[OwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .mcp_server_header_auth import McpServerHeaderAuth
from .mcp_server_o_auth2 import McpServerOAuth2
from .mcp_server_passthrough import McpServerPassthrough

McpServerAuth = typing.Union[McpServerHeaderAuth, McpServerOAuth2, McpServerPassthrough]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mime_type import MimeType


class BlobStorageReference(UniversalBaseModel):
    """
    A reference to content offloaded to blob storage
    """

    type: typing.Literal["blob-storage"] = "blob-storage"
    path: str = pydantic.Field()
    """
    Path to the content in blob storage
    """

    mime_type: MimeType = pydantic.Field()
    """
    MIME type of the content
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .external_blob_storage_source import ExternalBlobStorageSource
from .local_artifact_source import LocalArtifactSource
from .true_foundry_managed_source import TrueFoundryManagedSource

ArtifactManifestSource = typing.Union[TrueFoundryManagedSource, ExternalBlobStorageSource, LocalArtifactSource]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class McpServerIntegrationTransport(enum.StrEnum):
    """
    The transport protocol to use for communication with the MCP Server.
    """

    STREAMABLE_HTTP = "streamable-http"
    SSE = "sse"

    def visit(self, streamable_http: typing.Callable[[], T_Result], sse: typing.Callable[[], T_Result]) -> T_Result:
        if self is McpServerIntegrationTransport.STREAMABLE_HTTP:
            return streamable_http()
        if self is McpServerIntegrationTransport.SSE:
            return sse()



# This file was auto-generated by Fern from our API Definition.

import typing

from .aws_inferentia import AwsInferentia
from .gcp_tpu import GcpTpu
from .nvidia_gpu import NvidiaGpu
from .nvidia_miggpu import NvidiaMiggpu
from .nvidia_timeslicing_gpu import NvidiaTimeslicingGpu

ResourcesDevicesItem = typing.Union[NvidiaGpu, AwsInferentia, NvidiaMiggpu, NvidiaTimeslicingGpu, GcpTpu]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .subject_type import SubjectType


class Subject(UniversalBaseModel):
    subject_id: typing_extensions.Annotated[str, FieldMetadata(alias="subjectId")] = pydantic.Field()
    """
    Subject ID
    """

    subject_type: typing_extensions.Annotated[SubjectType, FieldMetadata(alias="subjectType")] = pydantic.Field()
    """
    Subject type
    """

    subject_slug: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="subjectSlug")] = (
        pydantic.Field(default=None)
    )
    """
    Subject slug
    """

    subject_display_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="subjectDisplayName")
    ] = pydantic.Field(default=None)
    """
    Subject display name
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .image_content_part_image_url import ImageContentPartImageUrl


class ImageContentPart(UniversalBaseModel):
    """
    Image content for the message
    """

    type: typing.Literal["image_url"] = pydantic.Field(default="image_url")
    """
    Type of the content part
    """

    image_url: ImageContentPartImageUrl = pydantic.Field()
    """
    Image URL linking to the image
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class AddonComponentName(enum.StrEnum):
    ARGOCD = "ARGOCD"
    ARGO_ROLLOUT = "ARGO_ROLLOUT"
    ARGO_WORKFLOW = "ARGO_WORKFLOW"
    TFY_AGENT = "TFY_AGENT"
    ISTIO_BASE = "ISTIO_BASE"
    ISTIO_DISCOVERY = "ISTIO_DISCOVERY"
    TFY_ISTIO_INGRESS = "TFY_ISTIO_INGRESS"
    AWS_LOAD_BALANCER_CONTROLLER = "AWS_LOAD_BALANCER_CONTROLLER"
    CERT_MANAGER = "CERT_MANAGER"
    KARPENTER = "KARPENTER"
    KARPENTER_CONFIG = "KARPENTER_CONFIG"
    PROMETHEUS = "PROMETHEUS"
    METRICS_SERVER = "METRICS_SERVER"
    KUBECOST = "KUBECOST"
    KEDA = "KEDA"
    AWS_EFS_CSI_DRIVER = "AWS_EFS_CSI_DRIVER"
    AWS_EBS_CSI_DRIVER = "AWS_EBS_CSI_DRIVER"
    TFY_GPU_OPERATOR = "TFY_GPU_OPERATOR"
    TFY_INFERENTIA_OPERATOR = "TFY_INFERENTIA_OPERATOR"
    ELASTI = "ELASTI"
    TFY_WORKFLOW_PROPELLER = "TFY_WORKFLOW_PROPELLER"
    GRAFANA = "GRAFANA"
    TRUEFOUNDRY = "TRUEFOUNDRY"
    TFY_PROMETHEUS_CONFIG = "TFY_PROMETHEUS_CONFIG"
    SPARK_OPERATOR = "SPARK_OPERATOR"
    TFY_LOGS = "TFY_LOGS"

    def visit(
        self,
        argocd: typing.Callable[[], T_Result],
        argo_rollout: typing.Callable[[], T_Result],
        argo_workflow: typing.Callable[[], T_Result],
        tfy_agent: typing.Callable[[], T_Result],
        istio_base: typing.Callable[[], T_Result],
        istio_discovery: typing.Callable[[], T_Result],
        tfy_istio_ingress: typing.Callable[[], T_Result],
        aws_load_balancer_controller: typing.Callable[[], T_Result],
        cert_manager: typing.Callable[[], T_Result],
        karpenter: typing.Callable[[], T_Result],
        karpenter_config: typing.Callable[[], T_Result],
        prometheus: typing.Callable[[], T_Result],
        metrics_server: typing.Callable[[], T_Result],
        kubecost: typing.Callable[[], T_Result],
        keda: typing.Callable[[], T_Result],
        aws_efs_csi_driver: typing.Callable[[], T_Result],
        aws_ebs_csi_driver: typing.Callable[[], T_Result],
        tfy_gpu_operator: typing.Callable[[], T_Result],
        tfy_inferentia_operator: typing.Callable[[], T_Result],
        elasti: typing.Callable[[], T_Result],
        tfy_workflow_propeller: typing.Callable[[], T_Result],
        grafana: typing.Callable[[], T_Result],
        truefoundry: typing.Callable[[], T_Result],
        tfy_prometheus_config: typing.Callable[[], T_Result],
        spark_operator: typing.Callable[[], T_Result],
        tfy_logs: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AddonComponentName.ARGOCD:
            return argocd()
        if self is AddonComponentName.ARGO_ROLLOUT:
            return argo_rollout()
        if self is AddonComponentName.ARGO_WORKFLOW:
            return argo_workflow()
        if self is AddonComponentName.TFY_AGENT:
            return tfy_agent()
        if self is AddonComponentName.ISTIO_BASE:
            return istio_base()
        if self is AddonComponentName.ISTIO_DISCOVERY:
            return istio_discovery()
        if self is AddonComponentName.TFY_ISTIO_INGRESS:
            return tfy_istio_ingress()
        if self is AddonComponentName.AWS_LOAD_BALANCER_CONTROLLER:
            return aws_load_balancer_controller()
        if self is AddonComponentName.CERT_MANAGER:
            return cert_manager()
        if self is AddonComponentName.KARPENTER:
            return karpenter()
        if self is AddonComponentName.KARPENTER_CONFIG:
            return karpenter_config()
        if self is AddonComponentName.PROMETHEUS:
            return prometheus()
        if self is AddonComponentName.METRICS_SERVER:
            return metrics_server()
        if self is AddonComponentName.KUBECOST:
            return kubecost()
        if self is AddonComponentName.KEDA:
            return keda()
        if self is AddonComponentName.AWS_EFS_CSI_DRIVER:
            return aws_efs_csi_driver()
        if self is AddonComponentName.AWS_EBS_CSI_DRIVER:
            return aws_ebs_csi_driver()
        if self is AddonComponentName.TFY_GPU_OPERATOR:
            return tfy_gpu_operator()
        if self is AddonComponentName.TFY_INFERENTIA_OPERATOR:
            return tfy_inferentia_operator()
        if self is AddonComponentName.ELASTI:
            return elasti()
        if self is AddonComponentName.TFY_WORKFLOW_PROPELLER:
            return tfy_workflow_propeller()
        if self is AddonComponentName.GRAFANA:
            return grafana()
        if self is AddonComponentName.TRUEFOUNDRY:
            return truefoundry()
        if self is AddonComponentName.TFY_PROMETHEUS_CONFIG:
            return tfy_prometheus_config()
        if self is AddonComponentName.SPARK_OPERATOR:
            return spark_operator()
        if self is AddonComponentName.TFY_LOGS:
            return tfy_logs()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .base_workbench_input import BaseWorkbenchInput
from .workbench_image import WorkbenchImage


class RStudio(BaseWorkbenchInput):
    """
    Describes the configuration for the Rstudio server
    """

    type: typing.Literal["rstudio"] = pydantic.Field(default="rstudio")
    """
    +value=rstudio
    """

    image: WorkbenchImage

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GitHelmRepo(UniversalBaseModel):
    type: typing.Literal["git-helm-repo"] = pydantic.Field(default="git-helm-repo")
    """
    +value=git-helm-repo
    """

    repo_url: str = pydantic.Field()
    """
    Git repository URL - Needs to be a valid URL.
    """

    revision: str = pydantic.Field()
    """
    Branch/Commit SHA/Tag of the git repo.
    """

    path: str = pydantic.Field()
    """
    Path to the chart.
    """

    value_files: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Helm values files for overriding values in the helm chart.
    The path is relative to the Path directory defined above
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .spark_job_java_entrypoint import SparkJobJavaEntrypoint
from .spark_job_python_entrypoint import SparkJobPythonEntrypoint
from .spark_job_python_notebook_entrypoint import SparkJobPythonNotebookEntrypoint
from .spark_job_scala_entrypoint import SparkJobScalaEntrypoint
from .spark_job_scala_notebook_entrypoint import SparkJobScalaNotebookEntrypoint

SparkJobEntrypoint = typing.Union[
    SparkJobPythonEntrypoint,
    SparkJobScalaEntrypoint,
    SparkJobJavaEntrypoint,
    SparkJobPythonNotebookEntrypoint,
    SparkJobScalaNotebookEntrypoint,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .base_o_auth2login_jwt_source import BaseOAuth2LoginJwtSource


class BaseOAuth2Login(UniversalBaseModel):
    """
    OAuth2 client configuration to get the JWT.
    """

    client_id: str = pydantic.Field()
    """
    client ID for OAuth2.
    """

    client_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Client secret or the TrueFoundry secret containing the client secret for OAuth2.
    """

    authorization_url: str = pydantic.Field()
    """
    URL for the authorization request
    """

    token_url: str = pydantic.Field()
    """
    The endpoint to exchange auth code for tokens.
    """

    scopes: typing.List[str] = pydantic.Field()
    """
    List of scopes to request from the OAuth2 provider.
    """

    jwt_source: BaseOAuth2LoginJwtSource = pydantic.Field()
    """
    Source of the JWT token to be used for verification.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alert_config_resource import AlertConfigResource
from .notification_target_for_alert_rule import NotificationTargetForAlertRule
from .prometheus_alert_rule import PrometheusAlertRule


class AlertConfig(UniversalBaseModel):
    type: typing.Literal["alert-config"] = pydantic.Field(default="alert-config")
    """
    Type
    """

    name: str = pydantic.Field()
    """
    Name - 3 to 35 lower case characters long alphanumeric word, may contain - in between, cannot start with a number
    """

    resource: AlertConfigResource
    notification_targets: typing.List[NotificationTargetForAlertRule] = pydantic.Field()
    """
    Configure one or more notification targets where alerts will be sent. Each target specifies a notification channel (email or slack).
    """

    rules: typing.List[PrometheusAlertRule] = pydantic.Field()
    """
    Define one or more alert rules that specify the conditions to monitor, when to trigger alerts, and how they should be handled.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ClusterManifestClusterType(enum.StrEnum):
    """
    The cluster type of this cluster
    """

    AWS_EKS = "aws-eks"
    GCP_GKE_STANDARD = "gcp-gke-standard"
    AZURE_AKS = "azure-aks"
    GENERIC = "generic"
    CIVO_TALOS = "civo-talos"

    def visit(
        self,
        aws_eks: typing.Callable[[], T_Result],
        gcp_gke_standard: typing.Callable[[], T_Result],
        azure_aks: typing.Callable[[], T_Result],
        generic: typing.Callable[[], T_Result],
        civo_talos: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClusterManifestClusterType.AWS_EKS:
            return aws_eks()
        if self is ClusterManifestClusterType.GCP_GKE_STANDARD:
            return gcp_gke_standard()
        if self is ClusterManifestClusterType.AZURE_AKS:
            return azure_aks()
        if self is ClusterManifestClusterType.GENERIC:
            return generic()
        if self is ClusterManifestClusterType.CIVO_TALOS:
            return civo_talos()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .library_name import LibraryName


class TransformersFramework(UniversalBaseModel):
    type: typing.Literal["transformers"] = pydantic.Field(default="transformers")
    """
    Type of the framework
    """

    library_name: typing.Optional[LibraryName] = pydantic.Field(default=None)
    """
    Name of the library for the framework
    """

    pipeline_tag: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `pipeline()` task this model can be used with e.g. `text-generation`. See [huggingface docs](https://huggingface.co/docs/transformers/main/en/main_classes/pipelines#transformers.pipeline.task) for all possible values
    """

    base_model: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base model Id from HuggingFace Hub. If this is a finetuned model, this points to the base model id used for finetuning.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .custom_guardrail_config_auth_data import CustomGuardrailConfigAuthData
from .custom_guardrail_config_operation import CustomGuardrailConfigOperation
from .custom_guardrail_config_target import CustomGuardrailConfigTarget


class CustomGuardrailConfig(UniversalBaseModel):
    """
    Custom
    """

    name: str = pydantic.Field()
    """
    The name of the Guardrail Config.
    """

    type: typing.Literal["integration/guardrail-config/custom"] = pydantic.Field(
        default="integration/guardrail-config/custom"
    )
    """
    +uiType=Hidden
    +value=integration/guardrail-config/custom
    """

    operation: CustomGuardrailConfigOperation = pydantic.Field()
    """
    The operation type to use for the Guardrail. Validate guardrails are used to validate requests and mutate can validate as well as mutate requests.
    Validate guardrails are run in parallel while mutate guardrails are run sequentially.
    """

    target: CustomGuardrailConfigTarget = pydantic.Field()
    """
    Specify whether the guardrail should be applied to the request or response. Guardrails with target "Request" can be only used in input guardrails and guardrails with target "Response" can only be used in output guardrails.
    """

    url: str = pydantic.Field()
    """
    The URL of the Guardrail to send a post request to.
    """

    auth_data: typing.Optional[CustomGuardrailConfigAuthData] = pydantic.Field(default=None)
    """
    Authentication data for the Guardrail Server.
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Headers for the Guardrail Server. Forwarded to the Guardrail Server as is. For example: `{"Authorization": "APIKey <token>"}`
    """

    config: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    The config for the Guardrail Server. This is a JSON object that will be sent as a config to Guardrail Server along with the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AwsInferentia(UniversalBaseModel):
    type: typing.Literal["aws_inferentia"] = pydantic.Field(default="aws_inferentia")
    """
    +value=aws_inferentia
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the AWS Inferentia Accccelerator. One of [INF1, INF2].
    This field is required for Node Selector and can be ignored in Nodepool Selector.
    """

    count: int = pydantic.Field()
    """
    Count of Inferentia accelerator chips to provide to the application
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .amqp_output_config import AmqpOutputConfig
from .core_nats_output_config import CoreNatsOutputConfig
from .kafka_output_config import KafkaOutputConfig
from .nats_output_config import NatsOutputConfig
from .sqs_output_config import SqsOutputConfig

WorkerConfigOutputConfig = typing.Union[
    SqsOutputConfig, NatsOutputConfig, CoreNatsOutputConfig, KafkaOutputConfig, AmqpOutputConfig
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PolicyFilters(UniversalBaseModel):
    """
    Filters
    """

    cluster_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Cluster names where Policy applies
    """

    workspace_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Workspace names where Policy applies
    """

    env_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Environment names where Policy applies
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .azure_key_auth import AzureKeyAuth
from .azure_pii_category import AzurePiiCategory
from .azure_pii_guardrail_config_domain import AzurePiiGuardrailConfigDomain


class AzurePiiGuardrailConfig(UniversalBaseModel):
    """
    Azure PII
    """

    name: str = pydantic.Field()
    """
    The name of the Guardrail Config.
    """

    type: typing.Literal["integration/guardrail-config/azure-pii"] = pydantic.Field(
        default="integration/guardrail-config/azure-pii"
    )
    """
    +uiType=Hidden
    +value=integration/guardrail-config/azure-pii
    """

    resource_name: str = pydantic.Field()
    """
    Name of your Azure AI Language resource where the PII detection service is deployed (e.g., my-language-resource)
    """

    api_version: str = pydantic.Field(default="2024-11-01")
    """
    API version for the PII detection API
    """

    custom_host: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom endpoint URL for the PII detection API (optional, uses default Azure endpoint if not specified)
    """

    operation: typing.Optional[typing.Literal["mutate"]] = pydantic.Field(default=None)
    """
    The operation type for this guardrail. Azure PII guardrails can only be used for mutate.
    """

    domain: AzurePiiGuardrailConfigDomain = pydantic.Field()
    """
    Specialized domain for PII detection. Use healthcare for PHI (Protected Health Information) or none for general text
    """

    pii_categories: typing.List[AzurePiiCategory] = pydantic.Field()
    """
    Categories of PII to detect.
    """

    model_version: str = pydantic.Field(default="latest")
    """
    Version of the PII detection model to use, use latest for the newest model or specify a specific version for consistency
    """

    language: str = pydantic.Field(default="en")
    """
    Language code for PII detection (e.g., en for English)
    """

    auth_data: AzureKeyAuth

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
